﻿#include <iostream>
#include <fstream>
#include <array>
#include <string>
#include <vector>
#include <map>
#include <locale>
#include <Windows.h>
#include <sstream>
#include <time.h>

using namespace std;

/* Struktúra definíciók */

// Egy adott karakterhez tartozó kép
struct SKarakter
{
	array<string, 7> sorok;
};

// Jármű nyílvántartási adatai
struct SJarmu
{
	string rendszam;
	string forgalmidatum;
	string szin;
	string tulajdonos;
	string helyseg;
	string cime;

	bool lejart() const
	{
		extern string maidatum();
		return forgalmidatum < maidatum();
	}
};

// operator>> az SJarmu struktúrához a jarmu.txt-ből való beolvasáshoz
istream& operator>>(istream& in, SJarmu& jarmu)
{
	getline(in, jarmu.rendszam, ';');
	getline(in, jarmu.forgalmidatum, ';');
	getline(in, jarmu.szin, ';');
	getline(in, jarmu.tulajdonos, ';');
	in >> jarmu.helyseg;
	getline(in, jarmu.cime);
	return in;
}

/* Tárolt adatok */
map<char, SKarakter> karakterek;
vector<string> rendszamok;
vector<SJarmu> jarmuvek;
vector<string> korozottek;

/* A program felületéhez szükséges függvények */
void fejlec(const string& str)
{
	int m = (73 - str.length()) / 2;
	cout << " " << string(m, '#') << "  " << str << "  " << string(m, '#') << " " << endl << endl;
}

// Várakozás billentyű lenyomásig
void tovabb()
{
	system("pause");
	system("cls");
}

// Két dátum között eltelt napok kiszámítása
int elteltNapok(string datum1, string datum2)
{
	if (datum1 < datum2)
		swap(datum1, datum2);

	tm time1, time2;
	memset(&time1, 0, sizeof time1);
	memset(&time2, 0, sizeof time2);

	time1.tm_year = atoi(&datum1[0]) - 1900;
	time1.tm_mon = atoi(&datum1[5]) - 1;
	time1.tm_mday = atoi(&datum1[8]);

	time2.tm_year = atoi(&datum2[0]) - 1900;
	time2.tm_mon = atoi(&datum2[5]) - 1;
	time2.tm_mday = atoi(&datum2[8]);

	auto t1 = mktime(&time1);
	auto t2 = mktime(&time2);

	return (t1 - t2) / 60 / 60 / 24;
}

// Mai dátum lekérése a felatban előírt formátumban
string maidatum()
{
	auto t = time(0);
	auto tm = gmtime(&t);
	char buffer[256];
	sprintf(buffer, "%04d.%02d.%02d", tm->tm_year + 1900, tm->tm_mon + 1, tm->tm_mday);
	return buffer;
}

// Karakterek képének beolvasása fájlból
bool karakterBeolvasas(const string& filename)
{
	ifstream f(filename);
	if (!f.is_open())
	{
		cout << filename << " beolvasása nem sikerült!" << endl;
		return false;
	}
	char c;
	while (f >> c && f.ignore(100, '\n'))
	{
		SKarakter karakter;
		for (int i = 0; i < 7; i++)
		{
			getline(f, karakter.sorok[i]);
		}
		karakterek[c] = karakter;
	}
	return true;
}

// Kötőjel karakter képének beolvasása fájlból
bool kotojelBeolvas(const string& filename)
{
	ifstream f(filename);
	if (!f.is_open())
	{
		cout << filename << " beolvasása nem sikerült!" << endl;
		return false;
	}
	SKarakter karakter;
	for (int i = 0; i < 7; i++)
	{
		getline(f, karakter.sorok[i]);
	}
	karakterek['-'] = karakter;
	return true;
}

// Rendszám leolvasása fotóról
// A különböző méretű fotók kezelésére kiszámoluk,
// hogy mennyivel lett felnagyítva a fotó.
// Ez a függvény működik eltérű vízszintes és függőleges
// nagyítás esetén is.
string rendszamLeolvasas(const vector<string>& foto)
{
	const int karaktermagassag = 7;
	const int karakterszelesseg = 5;
	const int alapszelessseg = karaktermagassag * karakterszelesseg + 6 * 2;
	int x_scale = foto[0].size();
	int y_scale = foto.size();
	if (x_scale % alapszelessseg || y_scale % karaktermagassag)
		return "";

	x_scale /= alapszelessseg; y_scale /= karaktermagassag;

	string rendszam = "";

	// 7 db karakter leolvasása
	for (int i = 0; i < 7; i++)
	{
		for (auto& karakter : karakterek)
		{
			// Megpróbáljuk ráilleszteni a karaktert a fotóra, azaz
			// ellenőrizzük, hogy a fotó megfelelő helyein a karakter
			// megfelelő képpontja áll
			bool egyez = true;
			for (int x = 0; x < karakterszelesseg; x++)
			{
				for (int y = 0; y < karaktermagassag; y++)
				{
					if (foto[y * y_scale][(i * (karakterszelesseg + 2) + x) * x_scale] != karakter.second.sorok[y][x])
					{
						egyez = false;
						break;
					}
				}
			}
			if (egyez)
				rendszam.push_back(karakter.first);
		}
	}
	return rendszam.size() == 7 ? rendszam : "";
}

// Rendszámfotók beolvasása
bool rendszamokBeolvasasa()
{
	for (int i = 10; i <= 29; i++)
	{
		auto filename = "tesztek\\" + to_string(i) + ".txt";
		ifstream f(filename);
		if (!f.is_open())
		{
			cout << filename << " beolvasása nem sikerült!" << endl;
			continue;
		}
		vector<string> foto; // Rendszámfotó
		string sor;
		while (getline(f, sor))
		{
			foto.push_back(sor);
		}
		string rendszam = rendszamLeolvasas(foto);
		rendszamok.push_back(rendszam);
	}
	return true;
}

// Körözött járművek rendszámainak beolvasása
bool korozottBeolvasas(const string& filename)
{
	ifstream f(filename);
	if (!f.is_open())
	{
		cout << filename << " beolvasása nem sikerült!" << endl;
		return false;
	}
	string rendszam;
	while (getline(f, rendszam))
	{
		korozottek.push_back(rendszam);
	}
	return true;
}

// Járművek adatainak beolvasása
bool jarmuBeolvasas(const string& filename)
{
	ifstream f(filename);
	if (!f.is_open())
	{
		cout << filename << " beolvasása nem sikerült!" << endl;
		return false;
	}

	f.ignore(500, '\n'); // Első sor kihagyása

	SJarmu jarmu;
	while (f >> jarmu)
	{
		jarmuvek.push_back(jarmu);
	}
	return true;
}

// Nyilvántartásban szereplő járműhöz tartozó adatok lekérése rendszám alapján
SJarmu *jarmuKeres(const string& rendszam)
{
	auto it = find_if(begin(jarmuvek), end(jarmuvek), [&](const SJarmu& jarmu)
	{
		return jarmu.rendszam == rendszam;
	});
	return it != end(jarmuvek) ? &*it : nullptr;
}

// Összesítés részfeladat
//   filename - kimeneti fájl neve
void osszesites(const string& filename)
{
	ofstream f(filename);
	if (!f.is_open())
	{
		cout << filename << " létrehozása nem sikerült!" << endl;
		return;
	}

	for (auto& rendszam : rendszamok)
	{
		if (!rendszam.empty())
		{
			f << rendszam;
			auto jarmu = jarmuKeres(rendszam);
			if (jarmu)
			{
				if (jarmu->lejart())
					f << ";nem ervenyes";
				else
					f << ";ervenyes";

				if (find(begin(korozottek), end(korozottek), rendszam) != end(korozottek))
					f << ";korozik";
				else
					f << ";nem korozik";
			}
			else
			{
				f << ";nincs nyilvántartva";
			}
		}
		f << endl;
	}
	cout << filename << " létrehozva!" << endl;
}

// Razzia helyszínek keresése
void razzia()
{
	// Helységek kigyűjtése és előfordulásaik megszámlálása
	map<string, int> helysegek;
	for (auto& rendszam : korozottek)
	{
		auto jarmu = jarmuKeres(rendszam);
		if (jarmu)
		{
			helysegek[jarmu->helyseg]++;
		}
	}

	// Legtöbbször előforduló helységek előfordulási számának megkeresése
	int legtobb = 0;
	for (auto& i : helysegek)
	{
		if (i.second > legtobb)
			legtobb = i.second;
	}

	// Kiírás
	if (legtobb <= 1)
	{
		cout << "Nincs kiemelt helyszín." << endl;
	}
	else
	{
		for (auto& i : helysegek)
		{
			if (i.second == legtobb)
				cout << i.first << endl;
		}
	}
}

// A színek biztonságosságának meghatározása
void biztosSzinek()
{
	// Színek kigyüjtése és előfordulásaik megszámlálása
	map<string, int> szinek;
	int korozottekszama = 0;
	for (auto& rendszam : korozottek)
	{
		auto jarmu = jarmuKeres(rendszam);
		if (jarmu)
		{
			szinek[jarmu->szin]++;
			korozottekszama++;
		}
	}

	// Százalékok kiszámítása
	vector<pair<string, int>> szazalekok;
	for (auto p : szinek)
	{
		p.second = 100 * p.second / korozottekszama;
		szazalekok.push_back(p);
	}

	// Adatok rendezése
	sort(begin(szazalekok), end(szazalekok), [](const pair<string, int>& p1, const pair<string, int>& p2)
	{
		if (p1.second == p2.second)
			return p1.first < p2.first;
		return p1.second > p2.second;
	});

	// Kiíratás
	for (auto& szin : szazalekok)
	{
		cout << szin.first << ": " << szin.second << "%" << endl;
	}
}

int main()
{
	locale::global(locale("")); // Ékezetes betűk miatt

	fejlec("Adatok beolvasása");
	if (!karakterBeolvasas("karakterek\\alapbetu.txt")
		|| !karakterBeolvasas("karakterek\\alapszam.txt")
		|| !kotojelBeolvas("karakterek\\vesszo.txt"))
		return 0;
	cout << "Karakterek beolvasva!" << endl;

	if (!jarmuBeolvasas("tesztek\\jarmu.txt"))
		return 0;

	cout << "Jarmu.txt beolvasva!" << endl;

	if (!korozottBeolvasas("tesztek\\korozott.txt"))
		return 0;
	cout << "Korozott.txt beolvasva!" << endl;
	tovabb();

	fejlec("Rendszámképek leolvasása");
	rendszamokBeolvasasa();
	for (const auto& rendszam : rendszamok)
	{
		cout << rendszam;
		if (!rendszam.empty() && !jarmuKeres(rendszam))
		{
			cout << " - nincs nyilvántartva";
		}
		cout << endl;
	}
	tovabb();

	fejlec("Lejárt forgalmi engedélyek");
	for (auto& rendszam : rendszamok)
	{
		auto jarmu = jarmuKeres(rendszam);
		if (jarmu && jarmu->lejart())
			cout << rendszam << " - " << elteltNapok(jarmu->forgalmidatum, maidatum()) << " napja lejárt" << endl;
	}
	tovabb();

	fejlec("Körözött autók");
	for (auto& rendszam : rendszamok)
	{
		if (find(begin(korozottek), end(korozottek), rendszam) != end(korozottek))
			cout << rendszam << endl;
	}
	tovabb();

	fejlec("Összesítés");
	osszesites("osszesit.txt");
	tovabb();

	fejlec("Razzia helyszínek");
	razzia();
	tovabb();

	fejlec("Biztonságos színek");
	biztosSzinek();
	tovabb();

	return 0;
}